function TDFR_MatMOL_DynamicRegridding(Pe,n0,zswitch,Reltol,Abstol)

%**************************************************************************
% The MatMol Group (2009)
% Function....... : Transient simulation of a jacketed tubular reactor with
%                   with dispersion and Danckwerts boundary conditions.
%                   "F. Logist, I. Smets, A. Vande Wouwer, J. Van Impe 2005.
%                   Optimal control of dispersive tubular chemical reactors:
%                   Part II" Procs of 16th IFAC World Congress, DVD-ROM, 6p.
% Technique...... : Solution with the MatMOL toolbox and dynamic regridding
%                   MOVGRID
% Input...........: Pe:       Peclet number
%                   n0:       number of vertex centered discritisation points 
%                   zswitch:  position where the jacket temperature switches 
%                             from its maximum to its minimum value 
%                   RelTol:   Relatitve integration tolerance
%                   AbsTol:   Absolute integration tolerance
%**************************************************************************
close all

global zL zR n ne ind X eq choice bloc0 bloc1 bloc2 bloc3
global alpha kappa mu tau
global u z dltz mon dltu x
global v D1 D2 Tin Cin beta1 delta1 Twmax Twmin E R k0 z1

if nargin < 1, Pe       = 1.0e+6;   end
if nargin < 2, n0       = 51;       end
if nargin < 3, zswitch  = 0.54;     end
if nargin < 4, Reltol   = 1.0e-4;   end
if nargin < 5, Abstol   = 1.0e-6;   end
     
tic % initialise clock


% 1 Parameters
% ************

v		=  0.1;                 % [m/s]     fluid velocity
L		=  1.0;                 % [m]       reactor length
Pe1	    =  Pe;					% [-]       Peclet number
Pe2	    =  Pe;					% [-]       Peclet number
D1		=  v*L/Pe1;				% [m^2/s]   dispersion coefficient
D2      =  v*L/Pe2;				% [m^2/s]   dispersion coefficient

Tin     =  340.0;               % [K]       feed temperature
Twmin   =  280.0;               % [K]       minimum jacket temperature
Twmax   =  400.0;               % [K]       maximum jacket temperature
Cin     =  0.02;                % [mol/L]   feed concentration

z1      =  zswitch;             % [m]       switching position
E		=  11250.0;             % [cal/mol] activation energy
R		=  1.986;               % [cal/(mol*K)] gass constant
k0		=  1.0e+6;              % [1/s]     rate constant       
     
% Note: 
% To avoid confusion with parameters from the regridding code, the reactor 
% related parameters beta and delta are given an additional index 1:
beta1	=	0.2;                % [1/s]     dimensionless heat transfer
delta1	=   0.25;               % [-]       dimensionless heat of reaction

% Spatial grid
n   	=   n0-2;               % [-]       # of moving grid points
zL      =   0.0;                % [m]       left border
zR      =   L;                  % [m]       right border
dz      =   (zR-zL)/(n+1);      % [m]       spatial discretisation length
z       =   [zL:dz:zswitch-dz/2 ... % [m]   position vector
            zswitch:dz:zR-dz/2 zR]'; 
% Note:
% If n0 is the original number of discretisation points (i.e., dim(z)), 
% n is the number of moving grid points. (z0 and zL are fixed.)

% Temporal grid
t_start =   0.0;                % [s]       initial time
t_end   =   20.0;               % [s]       final time
dt_plot =   1.0;                % [s]       time interval for plots
t_plot  =   t_start:dt_plot:t_end;  % [s]   time points to be plotted
     
% Parameters of the moving grid (MOVGRID)
ne      =   2;                  % [-]       nr of PDEs
alpha   =   0.1;                % [-]
kappa   =   2.0;                % [-]
mu      =   kappa*(kappa+1);    % [-]
tau     =   1.0e-7;             % [-]
choice  =   'der1-1';           % [-]       monitoring function choice


% 2 Numerical computation
% ***********************     

% 2a Initial conditions
X10 = Tin*ones(length(z),1);
X20	= 0.0*ones(length(z),1);
u   = [X10,X20];
for j=1:ne, % Reorder u to x(n+2,ne)
    x(j:ne+1:n*(ne+1))=u(2:n+1,j);
end
x(ne+1:ne+1:n*(ne+1))=z(2:n+1);
clear X10 X20     

% 2b Initialisation global variables for JPat
X = zeros(n*(ne+1),n*(ne+1));
eq(:,:,1) = [-2 2 1; 0 0 1];
eq(:,:,2) = [ 0 0 1;-2 2 1];

% 2c Initialisation general blocs of MvPattern
bloc0 = zeros(ne+1,ne+1);
bloc1 = bloc0;
bloc1(ne+1,:) = 1;
bloc2 = eye(ne+1);
bloc2(:,ne+1) = 1;
bloc2(ne+1,:) = 1;
bloc3 = bloc0;
bloc3(ne+1,ne+1) = 1;

% 2d Solution of the DAEs
options = odeset('RelTol',Reltol,'AbsTol',Abstol,'Mass',@mass,...
         'MStateDependence','strong','JPattern',JPat,'MvPattern',MvPat,...
         'MassSingular','no','BDF','on','MaxOrder',5);
sol  = ode15s(@ODE_MatMOL_DynamicRegridding,t_plot,x,options);
t    = t_plot;
sol2 = deval(sol,t);            % evaluate solution at time points
y    = sol2';
clear sol2

% 2e Reactor properties
% Separate the dependent variables and the node positions
Z = zeros(n+2,1);
T = zeros(n+2,1);
C = zeros(n+2,1);
for i=1:length(t),
    for j=1:n,
        Z(j+1,i) = y(i,(ne+1)*j);
        T(j+1,i) = y(i,(ne+1)*j-2);
        C(j+1,i) = y(i,(ne+1)*j-1);    
    end
    Z(1,i)   = zL;
    Z(n+2,i) = zR;
    dzg      = Z(2,i)-Z(1,i);
    % Danckwerts boundary conditions 
    T(1,i)   = (y(i,1)+v/D1*dzg*Tin)/(1+v/D1*dzg);
    C(1,i)   = (y(i,2)+v/D2*dzg*Cin)/(1+v/D2*dzg);
    T(n+2,i) = T(n+1,i)+(zR-Z(n+1,i))*(T(n+1,i)-T(n,i))/(Z(n+1,i)-Z(n,i));
    C(n+2,i) = C(n+1,i)+(zR-Z(n+1,i))*(C(n+1,i)-C(n,i))/(Z(n+1,i)-Z(n,i));
end

% 2e Integrator statisctics
Stats   = sol.stats; % solution statistics
CPUtime = toc;       % read clock
clear sol

disp('Integration statistics:')
fprintf('  %g successful steps\n', Stats.nsteps);
fprintf('  %g failed attempts\n',  Stats.nfailed);
fprintf('  %g function evaluations\n', Stats.nfevals);
fprintf('  %g partial derivatives\n',  Stats.npds);
fprintf('  %g LU decompositions\n', Stats.ndecomps);
fprintf('  %g solutions of linear systems\n', Stats.nsolves);
fprintf('  %g s (total) computation time\n',  CPUtime);


% 3 Plot results
% **************

figure(1)
subplot('position',[0.1 0.3 0.8 0.6])
plot(Z,T,'g',Z,T,'.k')
ylabel('Temperature T [K]','Fontsize',18)
title(['MatMOL Dynamic Regridding: Pe1=Pe2=' num2str(Pe2)],'Fontsize',18)
axis([zL zR 270.0 450.0])
set(gca,'Fontsize',16)

subplot('position',[0.1 0.1 0.8 0.15])
for i=1:length(t),
    plot(Z(:,i),t_plot(i)*ones(1,n+2),'.k')
    hold on
end
xlabel('Space z [m]','fontsize',18);
ylabel('Time [s]','Fontsize',18)
axis([zL zR t_start t_end])
set(gca,'Fontsize',16)
    
figure(2)
subplot('position',[0.1 0.3 0.8 0.6])
plot(Z,C,'g',Z,C,'.g')
ylabel('Concentration C [mole/K]','Fontsize',18)
title(['MatMOL Dynamic Regridding: Pe1=Pe2=' num2str(Pe2)],'Fontsize',18)
axis([zL zR -0.001 0.02])
set(gca,'Fontsize',16)
    
subplot('position',[0.1 0.1 0.8 0.15])
for i=1:length(t),
    plot(Z(:,i),t_plot(i)*ones(1,n+2),'.k')
    hold on
end
xlabel('Space z [m]','fontsize',18);
ylabel('Time [s]','Fontsize',18)
axis([zL zR t_start t_end])
set(gca,'Fontsize',16)

end % End of function TDFR_MatMOL_DynamicRegridding
% *************************************************************************

function xt = ODE_MatMOL_DynamicRegridding(t,x)
% Compute the right vector of the global ODEs system %

% The following code implements the right vector b(x,t) of the system
%
%            A(x,t)xdot = b(x,t)
%
% with
%          1   2         ne      1   2         ne            1         ne
%    x = [u   u   ...   u   z   u   u   ...   u   z   ...   u   ...   u   z ]'
%          1   1         1   1   2   2         2   2         n         n   n
%
% and where the meshpoints 1, 2, ... n are the interiors nodes. Nevertheless, b(x,t) uses also
%
% the boundary points and values z , z   , u  , u   .
%                                 0   n+1   0    n+1
%
% The code implements first  a) the monitor function M(i) :
%
%                                 j     j 2
%                               (u   - u )
%                         1  ne   i+1   i
%    M(i) = sqrt{alpha + -- sum --------   } , i = 0, ..., n
%                        ne j=1           2
%                               (z   - z )
%                                 i+1   i
%
%                            b) the "deltaz" vector dltz(i) :
%
%    dltz(i) = z   - z  , i = -1, ..., n+1 
%               i+1   i
%
% with   dltz(-1) = dltz(0)  and   dltz(n+1) = dltz(n).
%
% Following elements of b(x,t) are then implemented :
%
%    a) elements from the moving grid equation :
%
%            1         mu        1+2mu       mu          1         mu        1+2mu       mu
%    g(i) = --- [ - --------- + ------- - -------- ] - ----- [ - ------- + --------- - -------- ]
%           M(i)    dltz(i+1)   dltz(i)   dltz(i-1)    M(i-1)    dltz(i)   dltz(i-1)   dltz(i-2)
%
%    b) elements from the PDEs :
%
%    udot(i,j)   i = 1, ..., n       j = 1, ..., ne.
%
% Structure of b(x,t) : b(x,t) is (ne+1)*n vector :
%
%                udot(1,1)
%
%                udot(1,2)
%
%                . . .
%
%                udot(1,ne)
%
%                g(1)
%
%                udot(2,1)
%
%                udot(2,2)
%
%                . . .
%    b(x,t) =
%                udot(2,ne)
%
%                g(2)
%
%                . . . . . . . .
%
%                udot(n,1)
%
%                udot(n,2)
%
%                . . .
%
%                udot(n,ne)
%
%                g(n)
%
global zL zR n ne ind
global alpha kappa mu tau
global u z dltz mon dltu
global mo_inv dz_inv
global choice
global v D1 D2 Cin Tin beta1 delta1 Twmax Twmin E R k0 z1

t
% Separate dependent variables and node positions and implement the BCs
[u z] = BC_DynGrid(t,x);

% Compute the monitoring function mon(i)
mon = monitor(u,z,choice,t);

% Compute the right vector g(n) of the moving grid equation
g = rightmon(mon,dltz,mu);

% Spatial derivatives
Dzz = three_point_centered_D2(z);
fz  = kurg_centred_slope_limiter_fz(ne,n+2,z',t,u,'Flux_DynGrid',...
                            'dFlux_DynGrid_dx');

% Temporal derivatives
lim = find(z-z1<=0);
ind = lim(length(lim));
uw(1:ind,1)     = Twmax;
uw(ind+1:n+2,1) = Twmin;
    
udot(:,1) = D1*Dzz*u(:,1) - fz(:,1) + Tin/Cin*delta1*k0*u(:,2) ...
            .*exp(-E./(R*u(:,1))) + beta1.*(uw-u(:,1)); 
udot(:,2) = D2*Dzz*u(:,2) - fz(:,2) - k0*u(:,2).*exp(-E./(R*u(:,1)));     

% Reorder the global right vector
for j=1:ne,
      xt(j:ne+1:n*(ne+1),1)=udot(2:n+1,j);
end
xt(ne+1:ne+1:n*(ne+1),1)=g;

end % End of function ODE_MatMOL_DynamicRegridding
% *************************************************************************
%
% function Flux_DynGrid(n,ne,t,X)
% See separate m-file in the same directory
%
%**************************************************************************
%
% function dfxdx = dFlux_DynGrid_dx(n,ne,X)
% See separate m-file in the same directory
%
%**************************************************************************
%
% function BC_DynGrid(t,X)
% See separate m-file in the same directory
%
%**************************************************************************